# Copyright (C) 2019 Chintalagiri Shashank
#
# This file is part of Tendril.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Primitives for Discounts Support
--------------------------------
"""


from tendril.utils.types import ParseException
from tendril.utils.types.unitbase import Percentage
from tendril.utils.types.currency import CurrencyValue


class DiscountMixin(object):
    def apply_discount(self, discount, desc):
        if not isinstance(discount, (Percentage, CurrencyValue)):
            try:
                discount = CurrencyValue(discount)
            except ParseException:
                discount = Percentage(discount)
        self._discounts.append((desc, discount))

    @property
    def discounts(self):
        ep = self.base_price
        for name, discount in self._discounts:
            if isinstance(discount, Percentage):
                name = "{0} ({1})".format(name, discount)
                discount = ep * discount
            ep = ep - discount
            yield name, discount

    @property
    def effective_price(self):
        ep = self.base_price
        for _, discount in self.discounts:
            ep = ep - discount
        return ep

    def reset_dicounts(self):
        self._discounts = []



# Copyright (C) 2019 Chintalagiri Shashank
#
# This file is part of Tendril.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Structured Pricing Primitives
-----------------------------
"""

from tendril.utils.types.currency import CurrencyValue

from .base import PricingBase
from .tax import TaxMixin
from .addons import AddonMixin
from .discount import DiscountMixin


class StructuredUnitPrice(DiscountMixin, TaxMixin, PricingBase, AddonMixin):
    def __init__(self, *args, **kwargs):
        self._parent = kwargs.pop('parent', None)
        super(StructuredUnitPrice, self).__init__(*args, **kwargs)
        self._addons = []
        self._optional_addons = []
        self._discounts = []
        self._taxes = []

    def elements(self):
        e = super(StructuredUnitPrice, self).elements()
        e.update({'base':   self._p('base',   parser=CurrencyValue)})
        e.update(TaxMixin.elements(self))
        e.update(AddonMixin.elements(self))
        return e

    @property
    def base_price(self):
        return self.base

    def reset(self):
        PricingBase.reset_qty(self)
        AddonMixin.reset_addons(self)
        DiscountMixin.reset_dicounts(self)
        TaxMixin.reset_tax_rates(self)



# Copyright (C) 2019 Chintalagiri Shashank
#
# This file is part of Tendril.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Specialized Primitives for GST Support
--------------------------------------
"""


# TODO Move to configs
DEFAULT_TAX = [{'tax': 'CGST', 'rate': '9%'},
               {'tax': 'SGST', 'rate': '9%'},
               {'tax': 'IGST', 'rate': '0%'}]


class GSTMixin(object):
    def _get_tax(self, name):
        raise NotImplementedError

    def override_tax_rate(self, rate, tax=None):
        raise NotImplementedError

    def gst_set_national(self):
        sgst = self._get_tax('SGST')
        cgst = self._get_tax('CGST')
        if sgst.rate == 0:
            return
        if cgst.rate == 0:
            return
        self.override_tax_rate(cgst.rate + sgst.rate, tax='IGST')
        self.override_tax_rate(0,                     tax='CGST')
        self.override_tax_rate(0,                     tax='SGST')

    def gst_set_local(self):
        igst = self._get_tax('IGST')
        if igst.rate == 0:
            return
        self.override_tax_rate(igst.rate / 2, tax='SGST')
        self.override_tax_rate(igst.rate / 2, tax='CGST')
        self.override_tax_rate(0,             tax='IGST')
